# place your package dependencies here
package require Tk

# special packages can be placed in the profile
# directory under lib 

proc Init {root nspace} {
  variable zoom $root
  variable ns $nspace
  # Number of pixels in xy-direction
  variable i_PixNum 40 
  # Size of single pixel
  variable i_PixSize 10

  # Interactively rescale number of pixels
  frame $root.rescale -borderwidth 10
  pack $root.rescale -side top -fill x
  scale $root.rescale.pixnum -orient horizontal -from 15 -to 80 -variable ${ns}::i_PixNum
  button $root.rescale.pixok -text "Resize" -command ${ns}::resize
  pack $root.rescale.pixok -side right
  pack $root.rescale.pixnum -expand yes -fill x -side right 
  # Zoomcanvas
  canvas $root.zoomcan -height [expr {$i_PixNum * $i_PixSize}] -width [expr {$i_PixNum * $i_PixSize}] -background white
  #.can create image 0 0 $::XRlab::imgname
  pack $root.zoomcan
  
  # Draw Zoomgrid
  makegrid $i_PixNum $i_PixSize 
  
  # Apple aqua swaps middle and right buttons
  if {[tk windowingsystem]== "aqua"} {
    $::XRlab::can bind "XR" <2> +[list ${ns}::zoompix %x %y]
  } else {
    $::XRlab::can bind "XR" <3> +[list ${ns}::zoompix %x %y]
  }

  return "Zoom"
}

# Resize grid, after scale slider has been used
proc resize {} {
  variable i_PixNum
  variable i_PixSize
  variable zoom
  $zoom.zoomcan delete all
  makegrid $i_PixNum $i_PixSize 
}

# Draw the zoom grid with the given parameters
proc makegrid {pixnum pixsize} {
  variable zoom
  variable ns

  # Resize canvas
  $zoom.zoomcan configure -height [expr {$pixnum * $pixsize}] -width [expr {$pixnum * $pixsize}] -background white
  # Create grid
  for {set i 0} {$i<$pixnum} {incr i} {
    for {set j 0} {$j<$pixnum} {incr j} {
      $zoom.zoomcan create rectangle [expr {$i*$pixsize}] [expr {$j*$pixsize}] [expr {($i+1)*$pixsize}] [expr {($j+1)*$pixsize}] -fill blue -tag "r${i}_${j}"
      # Apple aqua swaps middle and right buttons
      if {[tk windowingsystem]== "aqua"} {
        $zoom.zoomcan bind "r${i}_${j}" <2> [list ${ns}::printcoords $i $j]
      } else {
        $zoom.zoomcan bind "r${i}_${j}" <3> [list ${ns}::printcoords $i $j]
      }
    }
  }
}

proc zoompix {x y} {
  variable zoom
  variable i_PixNum
  variable i_xOffset [expr {$x-int($i_PixNum/2)}]
  variable i_yOffset [expr {$y-int($i_PixNum/2)}]

  for {set i 0} {$i<$i_PixNum} {incr i} {
    for {set j 0} {$j<$i_PixNum} {incr j} {

      # Watch out for the picture borders    
      if { ($i_xOffset+$i >= 0 && $i_xOffset+$i < 512) && ($i_yOffset+$j >= 0 && $i_yOffset+$j < 512) } {
        # Rescale Color to fit into ZoomLow/ZoomHigh
        set d_Color [$::XRlab::imgname get [expr {$i_yOffset+$j}] [expr {$i_xOffset+$i}]] 
        # Do not divide by zero unless $Name == God
        if { [expr ($::XRlab::zoomhigh - $::XRlab::zoomlow) > 0] } {
          set d_Color [expr {double( $d_Color - $::XRlab::zoomlow) / ($::XRlab::zoomhigh - $::XRlab::zoomlow)}]
        } else {
          expr {$d_Color > $::XRlab::zoomhigh ? [set d_Color 1] : [set d_Color 0]}
        }
	# Rescale to 8bit Grayscale
	expr {$d_Color < 0 ? [set d_Color 0] : [expr {$d_Color > 1 ? [set d_Color 1] : 0} ] }
        set i_Color [expr {int(255 * $d_Color) }]
        # Put 8Bit color to actual gridbox
        $zoom.zoomcan itemconf "r${i}_${j}" -fill [format "#%02X%02X%02X" $i_Color $i_Color $i_Color] 
      # Outside Picture
      } else {
        $zoom.zoomcan itemconf "r${i}_${j}" -fill blue
      }
    }
  }
}

proc printcoords {i j} {
  variable i_xOffset
  variable i_yOffset

  if { ($i_xOffset+$i >= 0 && $i_xOffset+$i < 512) && ($i_yOffset+$j >= 0 && $i_yOffset+$j < 512) } {
    puts "Zoom: [expr {$i_xOffset+$i}] [expr {$i_yOffset+$j}] [$::XRlab::imgname get [expr {$i_yOffset+$j}] [expr {$i_xOffset+$i}]]"
  }
}
